from typing import Tuple, Set, Iterable, List


class GH_AbstractGraph:
    def add_GraphChanged(self, obj: GraphChangedEventHandler) -> None: ...
    def ClearCaches(self) -> None: ...
    def Draw_PostRenderGraph(self, g: Graphics, cnt: GH_GraphContainer) -> None: ...
    def Draw_PostRenderGrid(self, g: Graphics, cnt: GH_GraphContainer) -> None: ...
    def Draw_PostRenderGrip(self, g: Graphics, cnt: GH_GraphContainer, index: int) -> None: ...
    def Draw_PostRenderTags(self, g: Graphics, cnt: GH_GraphContainer) -> None: ...
    def Draw_PreRenderGraph(self, g: Graphics, cnt: GH_GraphContainer) -> GH_GraphDrawInstruction: ...
    def Draw_PreRenderGrid(self, g: Graphics, cnt: GH_GraphContainer) -> GH_GraphDrawInstruction: ...
    def Draw_PreRenderGrip(self, g: Graphics, cnt: GH_GraphContainer, index: int) -> GH_GraphDrawInstruction: ...
    def Draw_PreRenderTags(self, g: Graphics, cnt: GH_GraphContainer) -> GH_GraphDrawInstruction: ...
    def Duplicate(self) -> IGH_Graph: ...
    def EmitProxyObject(self) -> IGH_GraphProxyObject: ...
    def GDI_GraphPath(self, reg: RectangleF) -> Set(PointF): ...
    @property
    def Description(self) -> str: ...
    @property
    def GraphTypeID(self) -> Guid: ...
    @property
    def Grips(self) -> List: ...
    @property
    def Icon_16x16(self) -> Image: ...
    @property
    def IsValid(self) -> bool: ...
    @property
    def Name(self) -> str: ...
    def OnGraphChanged(self, bIntermediate: bool) -> None: ...
    def PrepareForUse(self) -> None: ...
    def Read(self, reader: GH_IReader) -> bool: ...
    def remove_GraphChanged(self, obj: GraphChangedEventHandler) -> None: ...
    def ValueAt(self, t: float) -> float: ...
    def Write(self, writer: GH_IWriter) -> bool: ...


class GH_BezierGraph(GH_AbstractGraph):
    def __init__(self): ...
    def ClearCaches(self) -> None: ...
    def Draw_PreRenderGrip(self, g: Graphics, cnt: GH_GraphContainer, index: int) -> GH_GraphDrawInstruction: ...
    def GDI_GraphPath(self, reg: RectangleF) -> Set(PointF): ...
    @property
    def GraphTypeID(self) -> Guid: ...
    @property
    def Icon_16x16(self) -> Image: ...
    @property
    def IsValid(self) -> bool: ...
    def Read(self, reader: GH_IReader) -> bool: ...
    def ValueAt(self, t: float) -> float: ...
    def Write(self, writer: GH_IWriter) -> bool: ...


class GH_ConicGraph(GH_AbstractGraph):
    def __init__(self): ...
    def GDI_GraphPath(self, reg: RectangleF) -> Set(PointF): ...
    @property
    def GraphTypeID(self) -> Guid: ...
    @property
    def Icon_16x16(self) -> Image: ...
    @property
    def IsValid(self) -> bool: ...
    def Read(self, reader: GH_IReader) -> bool: ...
    def ValueAt(self, t: float) -> float: ...
    def Write(self, writer: GH_IWriter) -> bool: ...


class GH_DoubleSineGraph(GH_AbstractGraph):
    def __init__(self): ...
    def ClearCaches(self) -> None: ...
    def Draw_PreRenderGraph(self, g: Graphics, cnt: GH_GraphContainer) -> GH_GraphDrawInstruction: ...
    def Draw_PreRenderGrip(self, g: Graphics, cnt: GH_GraphContainer, index: int) -> GH_GraphDrawInstruction: ...
    def GDI_GraphPath(self, reg: RectangleF) -> Set(PointF): ...
    @property
    def GraphTypeID(self) -> Guid: ...
    @property
    def Icon_16x16(self) -> Image: ...
    @property
    def IsValid(self) -> bool: ...
    def Read(self, reader: GH_IReader) -> bool: ...
    def ValueAt(self, t: float) -> float: ...
    def Write(self, writer: GH_IWriter) -> bool: ...


class GH_GaussianGraph(GH_AbstractGraph):
    def __init__(self): ...
    @property
    def GraphTypeID(self) -> Guid: ...
    @property
    def Icon_16x16(self) -> Image: ...
    @property
    def IsValid(self) -> bool: ...
    def Read(self, reader: GH_IReader) -> bool: ...
    def ValueAt(self, t: float) -> float: ...
    def Write(self, writer: GH_IWriter) -> bool: ...


class GH_GraphContainer:
    @overload
    def __init__(self, n_graph: IGH_Graph): ...
    @overload
    def __init__(self, n_graph: IGH_Graph, n_x0: float, n_x1: float, n_y0: float, n_y1: float): ...
    def add_GraphChanged(self, obj: GraphChangedEventHandler) -> None: ...
    def ClearCaches(self) -> None: ...
    def Duplicate(self) -> GH_GraphContainer: ...
    def FromX(self, t: float) -> float: ...
    def FromY(self, t: float) -> float: ...
    @property
    def DisplayScale(self) -> Single: ...
    @property
    def Graph(self) -> IGH_Graph: ...
    @property
    def LockGrips(self) -> bool: ...
    @property
    def Region(self) -> RectangleF: ...
    @property
    def X0(self) -> float: ...
    @property
    def X1(self) -> float: ...
    @property
    def Y0(self) -> float: ...
    @property
    def Y1(self) -> float: ...
    def OnGraphChanged(self, bIntermediate: bool) -> None: ...
    def PrepareForUse(self) -> None: ...
    def Read(self, reader: GH_IReader) -> bool: ...
    def RemapPointsToGraphRegion(self, pts: Set(PointF)) -> None: ...
    def remove_GraphChanged(self, obj: GraphChangedEventHandler) -> None: ...
    def Render_GraphBackground(G: Graphics, region: RectangleF, bActive: bool) -> None: ...
    def Render_GraphGrid(G: Graphics, region: RectangleF) -> None: ...
    def Render_GraphPen(self) -> Pen: ...
    def Render_GuidePen() -> Pen: ...
    def Render_HorizontalConstraint(g: Graphics, rec: RectangleF, t: float) -> None: ...
    def Render(self, G: Graphics, bIncludeDomainTags: bool, samples: List) -> None: ...
    def Render_VerticalConstraint(g: Graphics, rec: RectangleF, t: float) -> None: ...
    def RespondToKeyDown(self, sender: GH_Canvas, e: KeyEventArgs) -> GH_ObjectResponse: ...
    def RespondToKeyUp(self, sender: GH_Canvas, e: KeyEventArgs) -> GH_ObjectResponse: ...
    def RespondToMouseDoubleClick(self, sender: GH_Canvas, e: GH_CanvasMouseEvent) -> GH_ObjectResponse: ...
    def RespondToMouseDown(self, sender: GH_Canvas, e: GH_CanvasMouseEvent) -> GH_ObjectResponse: ...
    def RespondToMouseMove(self, sender: GH_Canvas, e: GH_CanvasMouseEvent) -> GH_ObjectResponse: ...
    def RespondToMouseUp(self, sender: GH_Canvas, e: GH_CanvasMouseEvent) -> GH_ObjectResponse: ...
    @DisplayScale.setter
    def DisplayScale(self, AutoPropertyValue: Single) -> None: ...
    @Graph.setter
    def Graph(self, Value: IGH_Graph) -> None: ...
    @LockGrips.setter
    def LockGrips(self, Value: bool) -> None: ...
    @Region.setter
    def Region(self, Value: RectangleF) -> None: ...
    @X0.setter
    def X0(self, Value: float) -> None: ...
    @X1.setter
    def X1(self, Value: float) -> None: ...
    @Y0.setter
    def Y0(self, Value: float) -> None: ...
    @Y1.setter
    def Y1(self, Value: float) -> None: ...
    def ToRegionBox(self, pt: PointF) -> PointF: ...
    def ToRegionBox_x(self, x: float) -> Single: ...
    def ToRegionBox_y(self, y: float) -> Single: ...
    def ToUnitBox(self, pt: PointF) -> PointF: ...
    def ToX(self, t_unit: float) -> float: ...
    def ToY(self, t_unit: float) -> float: ...
    def TryValueAt(self, t: float) -> float: ...
    def ValueAt(self, t: float) -> float: ...
    def Write(self, writer: GH_IWriter) -> bool: ...


class GH_GraphDrawInstruction:
    none = 0
    skip = 1


class GH_GraphGrip:
    @overload
    def __init__(self): ...
    @overload
    def __init__(self, nOther: GH_GraphGrip): ...
    @overload
    def __init__(self, nX: float, nY: float): ...
    @overload
    def __init__(self, nX: float, nY: float, nConstraint: GH_GripConstraint): ...
    def add_GripChanged(self, obj: GripChangedEventHandler) -> None: ...
    @property
    def Constraint(self) -> GH_GripConstraint: ...
    @property
    def Index(self) -> int: ...
    @property
    def Point(self) -> PointF: ...
    @property
    def X(self) -> float: ...
    @property
    def Y(self) -> float: ...
    def LimitToUnitDomain(self, bLimitX: bool, bLimitY: bool) -> None: ...
    def OnGripChanged(self, bIntermediate: bool) -> None: ...
    def op_Equality(A: GH_GraphGrip, B: GH_GraphGrip) -> bool: ...
    def op_Inequality(A: GH_GraphGrip, B: GH_GraphGrip) -> bool: ...
    def remove_GripChanged(self, obj: GripChangedEventHandler) -> None: ...
    @Constraint.setter
    def Constraint(self, Value: GH_GripConstraint) -> None: ...
    @X.setter
    def X(self, Value: float) -> None: ...
    @Y.setter
    def Y(self, Value: float) -> None: ...
    def SetIndex(self, nIndex: int) -> None: ...


class GH_GraphProxyObject:
    def __init__(self, n_owner: IGH_Graph): ...
    @property
    def Description(self) -> str: ...
    @property
    def IsValid(self) -> bool: ...
    @property
    def Name(self) -> str: ...




class GH_GripConstraint:
    none = 0
    horizontal = 1
    vertical = 2


class GH_LinearGraph(GH_AbstractGraph):
    def __init__(self): ...
    def Draw_PreRenderGraph(self, g: Graphics, cnt: GH_GraphContainer) -> GH_GraphDrawInstruction: ...
    def EmitProxyObject(self) -> IGH_GraphProxyObject: ...
    def GDI_GraphPath(self, reg: RectangleF) -> Set(PointF): ...
    @property
    def GraphTypeID(self) -> Guid: ...
    @property
    def Icon_16x16(self) -> Image: ...
    @property
    def IsValid(self) -> bool: ...
    def Read(self, reader: GH_IReader) -> bool: ...
    def SetFromParameters(self, nA: float, nB: float) -> None: ...
    def ValueAt(self, t: float) -> float: ...
    def Write(self, writer: GH_IWriter) -> bool: ...


class GH_LinearGraphProxy:
    def __init__(self, n_owner: GH_LinearGraph): ...
    @property
    def Intercept(self) -> float: ...
    @property
    def Slope(self) -> float: ...
    @Intercept.setter
    def Intercept(self, Value: float) -> None: ...
    @Slope.setter
    def Slope(self, Value: float) -> None: ...


class GH_ParabolaGraph(GH_AbstractGraph):
    def __init__(self): ...
    def Draw_PreRenderGraph(self, g: Graphics, cnt: GH_GraphContainer) -> GH_GraphDrawInstruction: ...
    @property
    def GraphTypeID(self) -> Guid: ...
    @property
    def Icon_16x16(self) -> Image: ...
    @property
    def IsValid(self) -> bool: ...
    def Read(self, reader: GH_IReader) -> bool: ...
    def ValueAt(self, t: float) -> float: ...
    def Write(self, writer: GH_IWriter) -> bool: ...


class GH_PerlinGraph(GH_AbstractGraph):
    def __init__(self): ...
    @property
    def GraphTypeID(self) -> Guid: ...
    @property
    def Icon_16x16(self) -> Image: ...
    @property
    def IsValid(self) -> bool: ...
    def Read(self, reader: GH_IReader) -> bool: ...
    def ValueAt(self, t: float) -> float: ...
    def Write(self, writer: GH_IWriter) -> bool: ...


class GH_PowerGraph(GH_AbstractGraph):
    def __init__(self): ...
    @property
    def GraphTypeID(self) -> Guid: ...
    @property
    def Icon_16x16(self) -> Image: ...
    @property
    def IsValid(self) -> bool: ...
    def Read(self, reader: GH_IReader) -> bool: ...
    def ValueAt(self, t: float) -> float: ...
    def Write(self, writer: GH_IWriter) -> bool: ...


class GH_SincGraph(GH_AbstractGraph):
    def __init__(self): ...
    def GDI_GraphPath(self, reg: RectangleF) -> Set(PointF): ...
    @property
    def GraphTypeID(self) -> Guid: ...
    @property
    def Icon_16x16(self) -> Image: ...
    @property
    def IsValid(self) -> bool: ...
    def Read(self, reader: GH_IReader) -> bool: ...
    def ValueAt(self, t: float) -> float: ...
    def Write(self, writer: GH_IWriter) -> bool: ...


class GH_SineEquation:
    def __init__(self): ...
    def Read(self, reader: GH_IReader) -> bool: ...
    def SetEquationFromGrips(self) -> None: ...
    def ValueAt(self, t: float) -> float: ...
    def Write(self, writer: GH_IWriter) -> bool: ...


class GH_SineGraph(GH_AbstractGraph):
    def __init__(self): ...
    def GDI_GraphPath(self, reg: RectangleF) -> Set(PointF): ...
    @property
    def GraphTypeID(self) -> Guid: ...
    @property
    def Icon_16x16(self) -> Image: ...
    @property
    def IsValid(self) -> bool: ...
    def Read(self, reader: GH_IReader) -> bool: ...
    def ValueAt(self, t: float) -> float: ...
    def Write(self, writer: GH_IWriter) -> bool: ...


class GH_SquareRootGraph(GH_AbstractGraph):
    def __init__(self): ...
    def Draw_PreRenderGraph(self, g: Graphics, cnt: GH_GraphContainer) -> GH_GraphDrawInstruction: ...
    @property
    def GraphTypeID(self) -> Guid: ...
    @property
    def Icon_16x16(self) -> Image: ...
    @property
    def IsValid(self) -> bool: ...
    def Read(self, reader: GH_IReader) -> bool: ...
    def ValueAt(self, t: float) -> float: ...
    def Write(self, writer: GH_IWriter) -> bool: ...


class GraphChangedEventHandler:
    def __init__(self, TargetObject: Object, TargetMethod: IntPtr): ...
    def BeginInvoke(self, sender: IGH_Graph, bIntermediate: bool, DelegateCallback: AsyncCallback, DelegateAsyncState: Object) -> IAsyncResult: ...
    def EndInvoke(self, DelegateAsyncResult: IAsyncResult) -> None: ...
    def Invoke(self, sender: IGH_Graph, bIntermediate: bool) -> None: ...


class GraphChangedEventHandler:
    def __init__(self, TargetObject: Object, TargetMethod: IntPtr): ...
    def BeginInvoke(self, sender: GH_GraphContainer, bIntermediate: bool, DelegateCallback: AsyncCallback, DelegateAsyncState: Object) -> IAsyncResult: ...
    def EndInvoke(self, DelegateAsyncResult: IAsyncResult) -> None: ...
    def Invoke(self, sender: GH_GraphContainer, bIntermediate: bool) -> None: ...


class GripChangedEventHandler:
    def __init__(self, TargetObject: Object, TargetMethod: IntPtr): ...
    def BeginInvoke(self, sender: GH_GraphGrip, bIntermediate: bool, DelegateCallback: AsyncCallback, DelegateAsyncState: Object) -> IAsyncResult: ...
    def EndInvoke(self, DelegateAsyncResult: IAsyncResult) -> None: ...
    def Invoke(self, sender: GH_GraphGrip, bIntermediate: bool) -> None: ...


class IGH_Graph:
    def add_GraphChanged(self, obj: GraphChangedEventHandler) -> None: ...
    def ClearCaches(self) -> None: ...
    def Draw_PostRenderGraph(self, g: Graphics, cnt: GH_GraphContainer) -> None: ...
    def Draw_PostRenderGrid(self, g: Graphics, cnt: GH_GraphContainer) -> None: ...
    def Draw_PostRenderGrip(self, g: Graphics, cnt: GH_GraphContainer, index: int) -> None: ...
    def Draw_PostRenderTags(self, g: Graphics, cnt: GH_GraphContainer) -> None: ...
    def Draw_PreRenderGraph(self, g: Graphics, cnt: GH_GraphContainer) -> GH_GraphDrawInstruction: ...
    def Draw_PreRenderGrid(self, g: Graphics, cnt: GH_GraphContainer) -> GH_GraphDrawInstruction: ...
    def Draw_PreRenderGrip(self, g: Graphics, cnt: GH_GraphContainer, index: int) -> GH_GraphDrawInstruction: ...
    def Draw_PreRenderTags(self, g: Graphics, cnt: GH_GraphContainer) -> GH_GraphDrawInstruction: ...
    def Duplicate(self) -> IGH_Graph: ...
    def EmitProxyObject(self) -> IGH_GraphProxyObject: ...
    def GDI_GraphPath(self, reg: RectangleF) -> Set(PointF): ...
    @property
    def Description(self) -> str: ...
    @property
    def GraphTypeID(self) -> Guid: ...
    @property
    def Grips(self) -> List: ...
    @property
    def Icon_16x16(self) -> Image: ...
    @property
    def IsValid(self) -> bool: ...
    @property
    def Name(self) -> str: ...
    def OnGraphChanged(self, bIntermediate: bool) -> None: ...
    def PrepareForUse(self) -> None: ...
    def remove_GraphChanged(self, obj: GraphChangedEventHandler) -> None: ...
    def ValueAt(self, t: float) -> float: ...


class IGH_GraphProxyObject:
    pass
